/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.plugin.develop;

import java.util.List;
import java.util.Map;

import org.opengion.hayabusa.develop.AbstractJspCreate;
import org.opengion.hayabusa.develop.JspConvertEntity;
import org.opengion.fukurou.xml.OGElement;

/**
 * query.jspの&lt;og:select&gt;タグを作成します。
 *
 * ●使用例
 *      JspConvertEntity e = new JspConvertEntity();
 *      e.setTableName("GF92");
 *      e.setColumnName("CLM");
 *      List< JspConvertEntity> a = new List< JspConvertEntity>();
 *      a.add( e );
 *      Map< String , List< JspConvertEntity> > m = new Map< String , List<JspConvertEntity> >();
 *      m.put( "ORDER", a );
 *      JspCreate  j = JspCreateFactory.newInstance( "SELECT",m );
 *      j.execute("&ltog:select /&gt");
 *
 * @author Takeshi.Takada
 *
 */
public class JspCreate_SELECT extends AbstractJspCreate {
	private List<JspConvertEntity> ORDER_ROWS ;
	private boolean IS_NULL ;

	/**
	 * 初期化メソッド
	 *
	 * 内部で使用する JspConvertEntity の リスト のマップを受け取り、初期化を行います。
	 *
	 * @og.rev 5.2.1.0 (2010/10/01) 名前空間を、og 決め打ちから、名前空間指定無しに変更します。
	 *
	 * @param	master	JspConvertEntityのリストのマップ
	 */
	@Override
	protected void init( final Map<String,List<JspConvertEntity>> master ) {
		ORDER_ROWS = master.get("ORDER");
//		IS_NULL = (ORDER_ROWS == null || ORDER_ROWS.isEmpty() );
		IS_NULL = !isNotEmpty( ORDER_ROWS );
//		KEY  = "og:select";
		KEY  = ":select";		// 5.2.1.0 (2010/10/01) 名前空間指定無し
		NAME = "query";
	}

	/**
	 * JSPに出力するタグの内容を作成します。
	 * 引数より作成前のタグの属性内容を確認するする事が出来ます。
	 *
	 * @og.rev 5.2.1.0 (2010/10/01) メソッドの引数を、OGAttributes から OGElement に変更します。
	 * @og.rev 5.2.1.0 (2010/10/01) 名前空間を、og 決め打ちから、引数を使用するように変更します。
	 *
	 * @param ele OGElementエレメントオブジェクト
	 * @param	nameSpace	このドキュメントのnameSpace( og とか mis とか )
	 *
	 * @return	変換された文字列
	 * @throws Throwable 変換時のエラー
	 */
	@Override
	protected String execute( final OGElement ele , final String nameSpace )  throws Throwable {
		if( IS_NULL ) { return ""; }

		String ns = (nameSpace.length() == 0) ? "" : nameSpace + ":" ;	// 5.2.1.0 (2010/10/01) 名前空間

		// TODO Auto-generated method stub
		//書き出す文字列を作成開始。
		StringBuilder sbTub = new StringBuilder();

		//JOIN情報から<og:select>タグの検索句を生成する準備をします。
//		sbTub.append( "\t\t<og:select name=\"ORDER_BY\" lbl=\"ORDER_BY\">").append( CR );
		sbTub.append( "\t\t<" ).append( ns ).append( "select name=\"ORDER_BY\" lbl=\"ORDER_BY\">").append( CR );
		boolean isFirst = true;

		for ( JspConvertEntity column : ORDER_ROWS ){
//			sbTub.append( "\t\t\t<og:option value=\"" );
			sbTub.append( "\t\t\t<" ).append( ns ).append( "option value=\"" );
			sbTub.append( column.getRemarks() );
			sbTub.append( "\"\t\t" );
			sbTub.append( " lbls=\"");
			sbTub.append( column.getRemarks() );
			sbTub.append( "\" ");
			if ( isFirst ){
				sbTub.append( "selected=\"selected\"");
				isFirst = false;
			}
			sbTub.append( " />" ).append( CR );
		}
//		sbTub.append( "\t\t</og:select>").append( CR );
		sbTub.append( "\t\t</" ).append( ns ).append( "select>").append( CR );

		return sbTub.toString();
	}

}
